uniform sampler2D geometryX;
uniform sampler2D geometryY;
uniform sampler2D geometryWeights;

uniform sampler2D channel1;
uniform sampler2D channel2;
uniform sampler2D channel3;

uniform sampler1D transferFunction;

uniform sampler2D arrowImage;

uniform int type;
uniform vec4 color;
uniform bool scale;
uniform float amount;
uniform float size;

uniform float blendAlpha;
uniform int colorMode;


float getInterpolated(sampler2D channel, vec2 coord){
	vec2 pos0 = vec2(texture2D(geometryX, coord).x, texture2D(geometryY, coord).x);
	vec2 pos1 = vec2(texture2D(geometryX, coord).y, texture2D(geometryY, coord).y);
	vec2 pos2 = vec2(texture2D(geometryX, coord).z, texture2D(geometryY, coord).z);
	vec3 weights = (texture2D(geometryWeights, coord).xyz*3.0)-1.0;
	//weights = vec3(1.05, 0.05, 0.05);
	float pos = (weights.x*(texture2D(channel, pos0).x)) + (weights.y*(texture2D(channel, pos1).x)) + (weights.z*(texture2D(channel, pos2).x));
	return pos;
}

vec4 getTransferfunctionColor(vec2 coord){
	float value = getInterpolated(channel3, coord);
	return texture1D(transferFunction, value);
}

void main() { 

	vec4 resultColor = gl_Color;

	if (type == 0){

		vec2 imgCoords = vec2(gl_TexCoord[6].x, gl_TexCoord[6].y);
	
		mat2 rotMatrix = mat2(gl_TexCoord[4].x,gl_TexCoord[4].y,-gl_TexCoord[4].y,gl_TexCoord[4].x);
		vec2 h = vec2(0.5, 0.5);
		if (scale){
			rotMatrix = rotMatrix*mat2(gl_TexCoord[4].z,0,0,gl_TexCoord[4].z);
		}
		vec2 lookup = (rotMatrix*(imgCoords-h))+h;
		if (lookup.x<0.0 || lookup.x>1.0 || lookup.y<0.0 || lookup.y>1.0){
			discard;
		}else {
		
			vec4 arrowColor = texture2D(arrowImage,lookup);
		
			if (arrowColor.a>0.01){
				resultColor = gl_Color*arrowColor;
			} else {
				discard;
			}
		}

	} else if (type == 1){
		
		if (colorMode == 0){
			resultColor = gl_Color*color;
		} else if(colorMode == 1){
			resultColor = gl_Color*getTransferfunctionColor(gl_TexCoord[0].xy);
		}
	
	} else if (type >= 2){

		resultColor = getTransferfunctionColor(gl_TexCoord[0].xy);
	
	}

	gl_FragColor = vec4(resultColor.xyz, resultColor.a*blendAlpha);
	
}


//gl_FragColor = vec4(gl_TexCoord[7].x,-gl_TexCoord[7].x,0,0);
//gl_FragColor = vec4(gl_TexCoord[7].x,0,0,0);
//gl_FragColor = texture1D(transferFunction, vec2(gl_TexCoord[7].x).x);

//gl_FragColor = texture1D(transferFunction, texture2D(channel1, gl_TexCoord[0].xy).x);

//vec2 imgCoords = vec2(gl_TexCoord[0].x, -gl_TexCoord[0].y);
//gl_FragColor = texture2D(arrowImage, imgCoords);
//gl_FragColor = gl_FrontColor;
