uniform sampler2D geometryX;
uniform sampler2D geometryY;
uniform sampler2D geometryWeights;

uniform sampler2D channel1;
uniform sampler2D channel2;
uniform sampler2D channel3;

uniform sampler1D transferFunction;

uniform int type;
uniform vec4 color;
uniform bool scale;

uniform int colorMode;

float getInterpolated(sampler2D channel, vec2 coord){
	vec2 pos0 = vec2(texture2D(geometryX, coord).x, texture2D(geometryY, coord).x);
	vec2 pos1 = vec2(texture2D(geometryX, coord).y, texture2D(geometryY, coord).y);
	vec2 pos2 = vec2(texture2D(geometryX, coord).z, texture2D(geometryY, coord).z);
	vec3 weights = (texture2D(geometryWeights, coord).xyz*3.0)-1.0;
	//weights = vec3(1.05, 0.05, 0.05);
	float pos = (weights.x*(texture2D(channel, pos0).x)) + (weights.y*(texture2D(channel, pos1).x)) + (weights.z*(texture2D(channel, pos2).x));
	return pos;
}

vec4 getTransferfunctionColor(vec2 coord){
	float value = getInterpolated(channel3, coord);
	return texture1D(transferFunction, value);
}

void main() {

	gl_Position = ftransform();
	
	gl_TexCoord[0] = gl_MultiTexCoord0;
	gl_TexCoord[6] = gl_MultiTexCoord6;

	if (type == 0){
		vec2 pos = vec2(texture2D(geometryX, gl_MultiTexCoord0.xy).x, texture2D(geometryY, gl_MultiTexCoord0.xy).x);

		float xValue = (getInterpolated(channel1, gl_MultiTexCoord0.xy))-0.5;
		float yValue = (getInterpolated(channel2, gl_MultiTexCoord0.xy))-0.5;

		float alpha = 0.0;

		alpha = atan(yValue, xValue);

		float scaleFactor = 1.0;
		if (scale){
			scaleFactor = 0.70710678/pow((xValue*xValue)+(yValue*yValue), 0.5);
		}

		float cosa = cos(alpha);
		float sina = sin(alpha);
		gl_TexCoord[4] = vec4(cosa,sina, scaleFactor,0);

		if (colorMode == 0){
			gl_FrontColor = color;
		} else if(colorMode == 1){
			gl_FrontColor = getTransferfunctionColor(gl_MultiTexCoord0.xy);
		}

	}else {
		gl_FrontColor = gl_Color;
	}
	gl_BackColor = gl_FrontColor;
}
